// Verilog test fixture for CountClear
// Feb 20, 2009

`timescale 1us / 100ns

module test_CountClear();

   reg CLK, RST;  
// Inputs to your CountClear from this test program
   wire GotStart, BIT9, FIX;

// Outputs from your CountClear to this test program
   wire ClrCOUNT8;

// Call (invoke) the UUT (Unit Under Test)
CountClear  UUT_CountClear (.ClrCOUNT8(ClrCOUNT8), 
               .GotStart(GotStart), .BIT9(BIT9), .FIX(FIX)  );


//--------------
/*  ***********  Variables of interest for debugging **********  *
This test bench contains code to generate all the signals in the circuit.
These signals are named   ...Corr. 
Example: GRAB9Corr = what GRAB9 should look like.
 The exception is samp[3:0] which does not end in Corr
Wen used as inputs for a module being tested, they are assigned to another veriable 
without the "Corr" sufix,  ie.
        assign  ... = ...Corr. . . 
Other useful variables:
. . SI = the serial input data without noise.
. . noise = A random binary number with a ErrRate % chance of being 1.
. . SerIn = SI XORed with noise.
. . ErrRate = 3% initially. Change this to increase/decrease noise.
. . timetick = A very rapid tick at the start of each data bit
               A triple width tick before the STOP bit.
. . samp[3:0] = the four last samples of SerIn taken at every rising CLK.
. . GotStart  = 1 when samp[3:0] = 0d00, 00d0, 000d or d000.
. . DATAVALID = BIT9 is high when parallel output data is valid.

. . Q2_Q1_Q0Corr = The three bits of the COUNT8 counter. To display in 
       ModleSim, Right click on the name and select ?  to expand.
. . data = The 8 data lines. DATA0 ...DATA7, collected together as a hex number
. . datacorrect = the hex number you should get from par2ser.
. . word = The transmitted MIDI word = {START 8-data-bits STOP}  Example: 0100101011
*/

integer Q2_Q1_Q0Corr;   initial Q2_Q1_Q0Corr=0; // Correct count for COUNT8;
   reg[0:7] datacorrect; initial datacorrect=0;
   reg [0:9] word;   // {STARTbit, data, STOPbit}
   real delaytime, cycletime;

   
// timetick can be displayed to give a 0.3us tick at the start of each data bit.
// It is 1.0us long at the start of the STOP bit.
   reg timetick;  initial timetick=0;

// Serial Data generation variables
   reg SI;  // Noise free serial data
   reg SerIn; initial SerIn=1; // SI^noise; MAJ is calculated from this.
   reg noise; // noise is 0 or 1 with ErrRate chance of being 1.
   initial noise=0;
   integer ErrRate; initial ErrRate = 3; // expressed in per cent.

// Initial seeding of random number generator.    
   integer seed; initial seed= 178363; //179637;
   reg [0:7] rand8bit; initial rand8bit=0; // used to hold 8 bits of random integer


// Samples of SerIn to get MAJ signal
// samp is a 4-bit shift reg which saves SerIn for the last 4 CLK cycles.
     reg [3:0] samp; reg MAJCorr; 
     initial begin samp=15; MAJCorr = 0; SI = 1; end

// Initialize circuit signals
   reg BIT8Corr, BIT9Corr, GotStartCorr, GRAB8Corr, GRAB9Corr, FIXCorr;  
   wire DATAVALID, ClrCOUNT8Corr;
   initial  begin BIT8Corr=0; GRAB9Corr=0; GRAB8Corr=0; BIT9Corr=1; 
         GotStartCorr=0; FIXCorr=0; end
   assign DATAVALID=BIT9Corr;




//  *******  Assign inputs here *************************
//  *** Test programs differ only here and in the top lines ****
//  Any inputs needed by the module under test should be assigned
//  to the appropriate ...Corr variable.   
      assign   BIT9=BIT9Corr;
      assign   FIX=FIXCorr;
      assign   GotStart=GotStartCorr;

// Outputs from the module under test are not used, but
// only to be manually compared with the ...Corr values.
// ************************************************************

//  For ser2par, collect the output as a vector so comparison with 
//  dataCorr is convenient.
//     wire  [0:7] data;  // 8 bits of data for each MIDI word.
//     assign   data={DATA7,DATA6,DATA5,DATA4,DATA3,DATA2,DATA1,DATA0};



// ******** CLK GENERATOR, NOISE ADDITION, FRONT END *********
// Clock, noise generation, and sampling routine.
 // Invert the CLK every 20 us
   initial CLK = 1;
   always 
     begin
     #17 // Move the noise edge off the clock edge.
         // get 1 from pos random numb mod 100 & less than ErrRate
         seed=Random(seed);
         noise = {seed}%100 < ErrRate; // {...}%100 keep noise positive
         SerIn=noise^SI; // SerIn elsewhere changes when SI changes
     #3 CLK = ~CLK;
         if (CLK == 1)
           begin
             // Fill shift reg with last 4 SI samples.
             samp = samp>>1;
             samp[3]=SerIn;
             MAJCorr=samp[1]&samp[2] | samp[2]&samp[3] | samp[1]&samp[3];
		   GotStartCorr = (~samp[0])&( (~samp[1])&(~samp[2]) |
                                    (~samp[2])&(~samp[3])  |
                                    (~samp[1])&(~samp[3])) |
                         (~samp[1])&(~samp[2])&(~samp[3]);
           end  
     end  // clock inversion

     // Remove delay between SI changing and SerIn changing
     always @(SI)  SerIn=SI;

     // Generate Clear Count8
     assign ClrCOUNT8Corr=(~GotStartCorr)&BIT9Corr&(~FIXCorr);

//  ************* End of CLK GENERATION *************



// ************** Startup RESET ***************
  initial
    begin
      RST = 0;
#1    RST = 1;   // Start reset after 1 us.
#50   RST = 0;   // Hold Reset for 50us then remove it.
  end
 

// **********  Loop to duplicate circuit behaviour *********
// Internal variables
    integer I;  initial I=0;           // I counts clock cycles

always
 begin
 // Loop to go through the 10 input bits, Start, 8 data, Stop.
   I=0; GRAB9Corr= 0;	   // <=== GRAB9Corr= 0, Added Mar 8 2011
   datacorrect=0;  
      wait (ClrCOUNT8Corr==0)
      while (ClrCOUNT8Corr==0)
          begin            
           @(posedge CLK)  // Wait for the clock edge, then go on.
             I=(I+1);      // I=1 should be middle of start bit
             Q2_Q1_Q0Corr = I%8;
              if (I==8) BIT9Corr=0;   //Middle of first bits. BIT9 is En controlled
              if (I%8>= 3) FIXCorr=1; else FIXCorr=0;
              if (I%8==7)  //Middle of all bits bit
                   GRAB9Corr =1; 
                 else  GRAB9Corr= 0;

		        if ( (I%8==0) & (I>7) ) begin
                     if (I<=64)  begin
                        datacorrect=datacorrect>>1;
                        datacorrect[0]=MAJCorr;
                        $display("MAJCorr=",MAJCorr,"  shift reg contents = %h",datacorrect);
                      end
                     else if (I<=72) begin
                        $display("MAJCorr=",MAJCorr, "  Stop bit\n");
                     end
                   end                      
              if ( (I>=(8*8))&(I<(8*9)) ) BIT8Corr =1;  //clk cycles 72 to 79
                 else BIT8Corr =0;

              if (I>=8*9) begin          //  <===Change Mar 4 2011
                         BIT9Corr=1;       //  <===Change Mar 4 2011
                         I=0;               // <===Added Mar 4 2011
                         end               // <===Added Mar 4 2011

        end  //while
        I=0;   Q2_Q1_Q0Corr = I%8;  // Duplicates COUNT8's 3 flip flops.
   end

//  ******** End of loop to duplicate circuit behaviour.  ***********


/* **************** Step through the MIDI words ************** */
// Generate the noise free data SI, inter word delays, transmiter clock
// and time ticks
initial
   begin

#201
  word = 10'b0_10101010_1; // first bit=0, START; last bit =1, STOP
  delaytime = 2000; // delay this time before START bit goes low.
  cycletime = 320;  //delay for data bit; about 8 CLK cycles.
  genSI (word, delaytime, cycletime); //, SI, timetick);  

delaytime = 977;
#1 seed = Random(seed);
   rand8bit = seed;  // Gets 8 least sig bits of seed.
   word ={1'b0,rand8bit,1'b1}; // concatenate in START and STOP bits..
   genSI (word, delaytime, cycletime);

#500
         $display($time, " Quitting time");
     $stop;  // avoids Model Sim asking if you are sure you want to exit Vsim

end 




/* ************ Generate SI ****************************** */

task    genSI; // (word, delaytime, cycletime)
// delaytime = delay from calling task to START bit going down.
// cycletime = length of data bit; normally 8*(clock period).
               // Global  (SI, timetick)
// Insert noisefree data SI and a time tick at the start
// of each data bit. A longer tick at the start of the STOP bit.
input [0:9] word; input delaytime, cycletime; 
real delaytime, cycletime;
// outputs SI, timetick; Keep these global so one gets response before end.
integer J;
begin         //task executables
   #delaytime
    $display("MIDIword = %h_%h_%h", word[0],word[1:8],word[9],
              "; word reversed = %h_%h_%h ",
     word[9], {word[8],word[7],word[6],word[5],word[4],word[3],word[2],word[1]}, word[0] );
      for (J = 0; J < 10; J = J + 1)
      begin
          SI = word[J];
          timetick = 1;
          // $display("J= %d ", J,"  SI= %b ", SI);
             if (J==9) begin  //STOP bit has a wider tick
               #1    timetick = 0;
               #(cycletime-1);
               end //(I==90)
             else    begin  // data bits have narrow tick
               #0.3 timetick = 0;
               #(cycletime -0.3);
             end //else 
    end  //for
  end //task executables       
endtask  //genSI


/* ************ Random ************* */
function integer Random;
// Silos $random always gave trouble. Also this will give the same
// sequence on most simulators.
     input seed; integer seed;
     begin
        Random=(seed*1723456789)>>4;
     end
endfunction


endmodule 

// *************************************************************

/* module CountClear(.ClrCOUNT8(ClrCOUNT8), 
                 .FIX(FIX), .GotStart(GotStart), .BIT9(BIT9) );

input BIT9, GotStart, FIX;
output ClrCOUNT8;
wire ClrCOUNT8, FIX;
assign ClrCOUNT8= (~GotStart) & (BIT9) & (~FIX) ;

endmodule   */



