// Verilog test fixture created for midi_top.sch - Sat Feb 27, 2010
// Here the MIDI data has some noise added.
`define random random(rand)
`timescale 1us / 100ns

module test_midi_noisy_top();

// --- Note: ModelSim seems to display variables in the order introduced. ---

// Signals generated in this test fixture.
   reg RST, CLK, SerIn;

// Signal used only to help find new midi words in the display.
   reg startguide; initial startguide=0;	
   // timetick displays 0.5us ticks at the start and end of each data bit.
   reg timetick;  initial timetick=0;

// Signals in lower level modules that will be teleported into test_top.
   wire MAJ; wire GRAB8; 
   assign MAJ = UUT2_midi_top.MAJ;
   assign GRAB8 = UUT2_midi_top.GRAB8;	
	
// Signals received from Midi_top by ports:
   wire DATA0, DATA1, DATA2, DATA3, DATA4, DATA5, DATA6, DATA7;
   wire CLK_OUT, DATAVALID;

// More signals in lower level modules that will be teleported into test_top.
     wire GRAB9;  assign GRAB9 = UUT2_midi_top.GRAB9;
	wire ClrSampCount, GotStart, FIX, BIT9;	
     assign ClrSampCount=UUT2_midi_top.ClrSampCount;
     assign GotStart=UUT2_midi_top.GotStart;
	assign FIX=UUT2_midi_top.FIX; 
	assign BIT9=UUT2_midi_top.BIT9; 
	  
// Signals used in this test fixture.
   reg  [7:0] serialin, serialinRev;
   wire [7:0] data, datarev;

// Counter bits will also be teleported to the top level.
     wire [2:0]  Q2Q1Q0;
	  assign Q2Q1Q0 = {UUT2_midi_top.Q2, UUT2_midi_top.Q1, UUT2_midi_top.Q0};
     wire   [3:0] Q3Q2Q1Q0a;
     assign Q3Q2Q1Q0a = {UUT2_midi_top.Q3a, UUT2_midi_top.Q2a, UUT2_midi_top.Q1a, UUT2_midi_top.Q0a};
// test holes
//   wire CE2a, D2a;
//	assign CE2a=UUT2_midi_top.BitCount.CE2a, 
// D2a=UUT_midi_top.BitCount.D2a;

   integer StudNumb1, StudNumb2, temp;
   initial temp=0;	

//ENTER YOUR STUDENT NUMBERs HERE. Ignore leading 100
//If you have no partner, leave the 2nd number as is.
//-------------------------------
   initial StudNumb1=654321;
   initial StudNumb2=030310;
//-------------------------------
/*
// Proceedure to get todays system date 
// http://verilog-interview-questions.blogspot.com/2009/08/how-to-display-system-date-in-display.html
// Declare an array 2 word deep 24 locations wide
reg [23:0] today [0:1];

initial
begin
$system("date +%m%d%y > date_file");
// output is 073199 for july 31st 1999
end 

*/
// ---------------------------------------
// I used with student number; J used in self test routine; K midi test words.
   integer I, J, K; initial K=0;
// xdata holds the data bits to be transmitted in this word, SerIn before adding noise.
// x holds the SerIn data without noise as read from xdata.
   reg [0:7] xdata;  initial xdata =8'B0000_0000;
   reg x;            initial x=1; // idle midi signal level

// noise is 4us period binary random noise. 
// When noise = 1, the data will be flipped for 4 us.
   reg noise; // noise is 0 or 1 with ErrRate chance of being 1.
   integer ErrRate; 
   initial ErrRate = 1; // expressed in per cent.
       // For the lab first try 4 percent errors. 15 is very noisy.
		 
   // Initial seeding of random number generator.    
   integer seed; initial seed= 139663; //179637;




//                           _______         _______
//Generate a clock      ____|       |_______|       |
initial CLK = 1;         // 0      2.0     4.0     6.0
always #2.0 CLK = ~CLK;              


initial begin
// Zero everything out
        SerIn=1;
        RST=0;
#0.5    RST=1;   // Raise RST to initialize all flip-flops to zero.
#5.2    RST=0;   // After 5.6us we are through with RST
#200;             // Wait 200us with SerIn=1, then will send a start bit below.
                  // This allows time to clear one MIDI Word


//=================MIDI word 1===============
/* Display the date, obtained from the system as above
   $readmemh("date_file", today);
// output is 073199 for july 31st 1999
   $display("Today is: %x", today[0]);
*/
// Do a warmup Midi command based on the rightmost 8 bits 
// of your student number expressed in binary. 
// Be sure your student number, or your partners, is entered at the top.
      $display("\n DATA: StudentNumber2 = %9d dec, %0h hex,", StudNumb2, StudNumb2);
      $display("\n DATA: StudentNumber1 = %9d dec, %0h hex,", StudNumb1, StudNumb1);
      $display(" Midi word is the rightmost two hex digits = %0h hex ", StudNumb1%256);
      
// Start bit
        SerIn=0;    startguide =1;
#32;                startguide=0;              
// 8 Data bits
       xdata=StudNumb1;
       for (I = 7; I >= 0; I=I-1)
         begin 
            SerIn=StudNumb1[I];  $display("StudNumb1[%1d] = %b", I,StudNumb1[I]);
          #32;
         end
        SerIn=1;  // Stop bit
//Output will be printed to log file after DATAVALID =1

//Delay 200 for stop and idle bits. Equivalent to #4*50
        repeat(50)    #4  SerIn= x^noise;
		  
// Fix it so the student numbers are frequently displayed.
        temp=StudNumb1; StudNumb1=StudNumb2; StudNumb2=temp; temp=0;

    
// Try some noisy data
// =============MIDI word 2; noisy word 1=============
// Start bit
        SerIn=0;    startguide=1;
#32;                startguide=0;              
// 8 Data bits
       xdata = 8'B1011_0100; //"_" ok in binary constants.
       SerIn=xdata[0];  //D0 
         repeat(8)
           begin  // noise gives ErrRate% chance of flipping bit
              #4  SerIn= x^noise; //x is collected in parallel monitor
           end
#0;  // Keep delay per bit at 32 us
       SerIn=xdata[1]; //D1
         repeat(8)    #4  SerIn= x^noise; 
#0; 
       SerIn=xdata[2]; //D2
         repeat(8) #4  SerIn= x^noise; 
#0; 
       SerIn=xdata[3]; //D3
         repeat(8)   #4  SerIn= x^noise; 
#0;  
       SerIn=xdata[4]; //D4
         repeat(8)   #4  SerIn= x^noise; 
#0;    
       SerIn=xdata[5]; //D5
         repeat(8)   #4  SerIn= x^noise;
#0;  
       SerIn=xdata[6]; //D6
         repeat(8)    #4  SerIn= x^noise;
#0;
       SerIn=xdata[7];  //D7
         repeat(8)   #4  SerIn= x^noise;
#0;    
// Stop Bit
       SerIn=1;
//Output will be displayed on the log file after DATAVALID =1
  //Delay 200 for stop and idle bits. Equivalent to #4*50
        repeat(50)    #4  SerIn= x^noise;
		  
// Fix it so the student numbers are frequently displayed.
        temp=StudNumb1; StudNumb1=StudNumb2; StudNumb2=temp; temp=0;


// ==========3rd word, 2nd noisy word============
// Start bit
        SerIn=0;    startguide=1;
#32;                startguide=0;              
// 8 Data bits
       xdata = 8'B0110_1110;
       SerIn=xdata[0]; // D0  
         repeat(8)
           begin  // noise gives  ErrRate% chance of flipping bit
              #4  SerIn= x^noise; //x is collected in parallel monitor
           end
       
#0;    // Keep delay per bit at 32 us
       SerIn=xdata[1];  //D1
         repeat(8)    #4  SerIn= x^noise; 
#0;    
       SerIn=xdata[2]; //D2 
         repeat(8)    #4  SerIn= x^noise;
#0; 
       SerIn=xdata[3]; //D3
         repeat(8)    #4  SerIn= x^noise; 
#0;    
       SerIn=xdata[4]; //D4 
         repeat(8)   #4  SerIn= x^noise;
#0;  
      SerIn=xdata[5]; //D5 
         repeat(8)   #4  SerIn= x^noise;
#0
       SerIn=xdata[6]; //D6
         repeat(8)   #4  SerIn= x^noise;
#0;
       SerIn=xdata[7]; //D7
         repeat(8)    #4  SerIn= x^noise;  
#0;  // Stop Bit
       SerIn=1;
//Output will be displayed on the log file after DATAVALID =1
  //Delay 200 for stop and idle bits. Equivalent to #4*50
        repeat(50)    #4  SerIn= x^noise;  
		  
		  
// Fix it so the student numbers are frequently displayed.
        temp=StudNumb1; StudNumb1=StudNumb2; StudNumb2=temp; temp=0;

// ==========4rd word, 3nd noisy word============
// Start bit
        SerIn=0;    startguide=1;
#32;                startguide=0;              
// 8 Data bits
       xdata = 8'B1110_0110;
       SerIn=xdata[0]; // D0    
         repeat(8)
           begin  // noise gives  ErrRate% chance of flipping bit
              #4  SerIn= x^noise; //x is collected in parallel monitor
           end
       
#0;    // Keep delay per bit at 32 us
       SerIn=xdata[1];  //D1
         repeat(8)   #4  SerIn= x^noise; 
#0;    
       SerIn=xdata[2]; //D2 
         repeat(8)    #4  SerIn= x^noise;
#0; 
       SerIn=xdata[3]; //D3
         repeat(8)   #4  SerIn= x^noise; 
#0;    
       SerIn=xdata[4]; //D4 
         repeat(8)   #4  SerIn= x^noise;
#0;  
      SerIn=xdata[5]; //D5 
         repeat(8)   #4  SerIn= x^noise;
#0
       SerIn=xdata[6]; //D6 
         repeat(8)   #4  SerIn= x^noise;
#0;
       SerIn=xdata[7]; //D7
         repeat(8)   #4  SerIn= x^noise;
#0;    
// Stop Bit
       SerIn=1;
		 
// Fix it so the student numbers are frequently displayed.
        temp=StudNumb1; StudNumb1=StudNumb2; StudNumb2=temp; temp=0;
		  
//Output will be displayed on the log file after DATAVALID =1
#150;   //Delay for stop and idle bits.
$display("\n You should have four MIDI words. If not, try Zoom out full."); 
$display( "That is the zoom with the filled in circle \n" );
     $stop;  //Give at least two cycles to generate datavalid

end  //initial   
//
//
//
//
//
//


/////////////////////// What is CLK_OUT for?
// Instantiate (invoke) the UUT (unit under test)
midi_top  UUT2_midi_top( .CLK(CLK), .RST(RST), .SerIn(SerIn),
             .BIT9(DATAVALID), 
             .P0(DATA0), .P1(DATA1), .P2(DATA2), .P3(DATA3), 
             .P4(DATA4), .P5(DATA5), .P6(DATA6), .P7(DATA7),
             .CLK_OUT(CLK_OUT)     );


//     - - - - - - - - - - -
// Little routine which runs in parallel and
// collects the serial input data for displaying on output log.
always@(posedge startguide)
  begin: CollectSerialIn
     K=K+1;
     x=0;   //Make noisefree data show start bit
   #32;
     #1 serialin=0; serialinRev=0;
     
     for ( J=7; J>=0; J=J-1)
       begin
            x=xdata[7-J];    //get the original starting value for noise generator 
            serialinRev = {x,serialinRev[7:1]};
            serialin = serialin <<1;    serialin[0] =x; 
            timetick=1; // timetick can be displayed to box words.
          #0.5;  timetick=0;  #0.5;
          #31;
       end
          x = SerIn;   // Capture stop bit to make display look consistant
          timetick =1;
          #1 timetick=0; // Longer tick marking stop bit.
         

 //    @(posedge DATAVALID)
 //    #5     // Get away from the clock edge
     $display("\n Midi word number %1d: ", K); 
     $display("    Serial data = %h hex (%b_%b_%b bin), If subscripts reversed = %b_%b_%b bin",
                  serialin, 1'b0,serialin[7:0],1'b1, 1'b1,serialinRev[7:0],1'b0);

      //  $display("    Serial data = %h hex (%b bin), If subscripts reversed = %b bin",
      //               serialin, serialin, serialinRev);
     $display("    Parallel data = %h hex [0_%b_1 bin]; or rev is %h hex [0_%b_1 bin]", data, data, datarev, datarev); 
  end  //CollectSerialIn

  assign datarev={DATA7, DATA6, DATA5, DATA4, DATA3,
                DATA2, DATA1, DATA0}; 
  assign data={DATA0, DATA1, DATA2, DATA3, DATA4,
  
                DATA5, DATA6, DATA7}; 

//     - - - - - - - - - -
// Random noise generating routine. Gives ErrRate% chance of a 1 which causes an error.
// If ErrRate=20, there will be 20% errors.  
always
     begin
	   seed = Random(seed);
      if (seed > 0) begin
    #4   if ((seed) %100 < ErrRate) noise = 1;
         else noise = 0;
     end
 end
	  
	  /* ************ Random ************* */
function integer Random;
// Silos $random always gave trouble. I also had trouble with ModleSim
//Also this will give the same sequence on most simulators.
     input seed; integer seed;
     begin
        Random=(seed*1723456789)>>2;
     end
endfunction


endmodule

//--------------------------------------------------------
/* 
module midi_top ( CLK, RST, SerIn, BIT9, P0,P1,P2,P3,P4,P5,P6,P7, CLK_OUT);
     
input CLK, RST, SerIn;

output  P0,P1,P2,P3,P4,P5,P6,P7;
//  DATA7, DATA6, DATA5, DATA4, DATA3, DATA2, DATA1, DATA0;
output CLK_OUT, BIT9; //DATAVALID;
wire CLK_OUT;  //The CLK feeds through and comes out as CLK_OUT
wire Samp0, Samp1, Samp2, Samp3;  // the last 4 samples of SerIn
wire   Q0, Q1, Q2, Q0a, Q1a, Q2a, Q3a; //Flipflop outputs in SampCount, BitCount
wire BIT8, BIT9, GRAB9, ClrSampCount, MAJ,GotStart;
wire GRAB8;

assign CLK_OUT = CLK;
//assign DATAVALID = BIT9;
assign GRAB8=UUT2_ser2par.GRAB8; //  ClrSampCount=UUT2_SampCount.ClrSampCount;


Last4Samp   UUT2_Last4Samp (.CLK(CLK), .RST(RST), .SerIn(SerIn), 
	      .Samp0(Samp0), .Samp1(Samp1), .Samp2(Samp2), .Samp3(Samp3));

FindStart UUT2_FindStart ( .GotStart(GotStart), 
	      .Samp0(Samp0), .Samp1(Samp1), .Samp2(Samp2), .Samp3(Samp3));

MajorityCir UUT2_MajorityCir ( .MAJ(MAJ), 
           .Samp3(Samp3), .Samp2(Samp2), .Samp1(Samp1));

CountClear  UUT2_CountClear (.CLK(CLK), .RST(RST), .ClrSampCount(ClrSampCount), 
               .GotStart(GotStart), .BIT8(BIT8), .BIT9(BIT9), .FIX(FIX)  );

BitCount    UUT2_BitCount ( .CLK(CLK), .RST(RST),
                .GRAB9(GRAB9),  .BIT8(BIT8), 
                .BIT9(BIT9),  .Q0a(Q0a), .Q1a(Q1a),
                .Q2a(Q2a), .Q3a(Q3a) );


SampCount  UUT2_SampCount ( .CLK(CLK), .RST(RST), .ClrSampCount(ClrSampCount),
                .GRAB9(GRAB9), .FIX(FIX),
                .Q0(Q0), .Q1(Q1), .Q2(Q2) );
// Some circuits may calculate GRAB8 in MidiTop; This calculates it inside ser2par
 
ser2par   UUT2_ser2par (.CLK(CLK), .RST(RST), .MAJ(MAJ), 
                     .DATA7(P7),  .DATA6(P6),
                     .DATA5(P5),  .DATA4(P4),
                     .DATA3(P3),  .DATA2(P2), 
                     .DATA1(P1),  .DATA0(P0), 
                     .GRAB9(GRAB9), .BIT8(BIT8) );

endmodule


//--------------------------------------------------------

module Last4Samp (.CLK(CLK), .RST(RST), .SerIn(SerIn), 
	  .Samp0(Samp0), .Samp1(Samp1), .Samp2(Samp2), .Samp3(Samp3));

input  SerIn, CLK, RST;
output Samp0, Samp1, Samp2, Samp3;
wire   Samp0, Samp1, Samp2, Samp3;
reg [3:0] Samp;

assign Samp0= Samp[0], Samp1= Samp[1], Samp2= Samp[2], Samp3= Samp[3];

always@(posedge CLK or posedge RST)
    if (RST) Samp <= 0;
    else
       begin
         Samp <= Samp>>1;
         Samp[3] <= SerIn;
       end
endmodule


//--------------------------------------------------------
module FindStart ( .GotStart(GotStart), 
	      .Samp0(Samp0), .Samp1(Samp1), .Samp2(Samp2), .Samp3(Samp3));

input  Samp0, Samp1, Samp2, Samp3;
wire   Samp0, Samp1, Samp2, Samp3;
output GotStart;
wire   GotStart;

assign GotStart=  ~Samp0&(~Samp1)&(~Samp3) | ~Samp0&(~Samp2)&(~Samp3) |
                  ~Samp0&(~Samp1)&(~Samp2) | ~Samp1&(~Samp2)&(~Samp3);

endmodule


//--------------------------------------------------------
module MajorityCir ( .MAJ(MAJ), 
           .Samp3(Samp3), .Samp2(Samp2), .Samp1(Samp1));
input  Samp2, Samp3, Samp1;
wire   Samp2, Samp3, Samp1;
output MAJ;
wire   MAJ;

assign MAJ =  Samp2&Samp3 | Samp2&Samp1 | Samp3&Samp1 ;

endmodule


//--------------------------------------------------------
module CountClear(.CLK(CLK), .RST(RST),
                 .ClrSampCount(ClrSampCount), .FIX(FIX), 
                 .GotStart(GotStart), .BIT8(BIT8), .BIT9(BIT9) );

input CLK, RST, BIT8, BIT9, GotStart, FIX;
output ClrSampCount;
wire ClrSampCount, GotStart;
assign ClrSampCount= (~GotStart) & (BIT9) & (~FIX) ;

endmodule

//------------------------------------------------------
module BitCount( .CLK(CLK), .RST(RST), 
                .GRAB9(GRAB9),  .BIT8(BIT8), 
                .BIT9(BIT9),
                .Q0a(Qa[0]), .Q1a(Qa[1]), .Q2a(Qa[2]), .Q3a(Qa[3]) );

input CLK, RST, GRAB9;
output BIT8, BIT9;
wire  BIT8, BIT9;
output [3:0] Qa;
reg [3:0] Qa;

always @(posedge CLK or posedge RST)
  if (RST) Qa<=0;
  else if (GRAB9) 
       begin
         Qa <= (Qa+1)%9;

       end

assign   BIT8 = Qa==8;  // ~((Qa>=1)&(Qa<9));
assign   BIT9 = Qa==0; // ~((Qa>=1)&(Qa<10));


endmodule

//-------------------------------------------------


module SampCount ( .CLK(CLK), .RST(RST), .ClrSampCount(ClrSampCount),
                .GRAB9(GRAB9), .FIX(FIX),
                .Q0(Q[0]), .Q1(Q[1]), .Q2(Q[2])  );
input CLK, RST, ClrSampCount;
wire ClrSampCount;
output GRAB9, FIX;
output [2:0] Q; // holds the count.
reg [2:0] Q;
wire [2:0] nextcount;
wire GotStart;

   always@(posedge CLK or posedge RST)
      if (RST) Q=0;
      else Q=nextcount;

 assign GRAB9= (Q==7);
 assign nextcount=(Q+1)&{3{~(ClrSampCount)}};
 assign FIX = ( Q >= 4 );
 
endmodule



//------------------------------------------------

module ser2par(.DATA0(DATA0), .DATA1(DATA1), .DATA2(DATA2), 
                .DATA3(DATA3), .DATA4(DATA4), .DATA5(DATA5), 
                .DATA6(DATA6), .DATA7(DATA7), 
                .MAJ(MAJ), .CLK(CLK), .RST(RST), 
                .GRAB9(GRAB9), .BIT8(BIT8)  );
 
input  MAJ, CLK, RST, GRAB9, BIT8;
output DATA0, DATA1, DATA2, DATA3, DATA4, DATA5, DATA6, DATA7;
wire   DATA0, DATA1, DATA2, DATA3, DATA4, DATA5, DATA6, DATA7;
wire BIT8, GRAB8;
reg [7:0] data;

 assign DATA7= data[7], DATA6= data[6], DATA5= data[5], DATA4= data[4],
   DATA3= data[3], DATA2= data[2], DATA1= data[1], DATA0= data[0];
 assign GRAB8=GRAB9&(~BIT8);

 always@(posedge CLK or posedge RST)
    if (RST) data <= 0;
    else if (GRAB8) 
       begin
         data <= data>>1;
         data[7] <= MAJ;
       end

endmodule
*/
